/*
hmr.c

This program read metadata from archive HDF file and compare 
it with inventory metadata. Then update and write metadata 
back to the HDF file.

Zhangshi Yin
 
2002.1.2.
*/


#include "mfhdf.h"
#include <stdio.h>
#include <stdlib.h>


/* Read metadata from hdfName and write temp file. */
int readMetadata(char *hdfName, FILE *metadataFile, char *attr);

/* Read metadata from hdfFile and write temp file. */
intn readAttr(int32 hdfFile, FILE *metadataFile, char *attr);


main(int argc, char *argv[])
{
  intn status;
  char *hdfName, *metadataName, oLine[1024], aLine[1024], tempLine[1024];
  char inLine[1024], outLine[1024], cmdstr[1024], attribute[1024];
  FILE *metadataFile, *tempArchiveFile, *tempInventoryFile;

  /* Check for arguments. */
  if (argc !=3) 
  {
    printf("\n");
    printf("Usage: hmr inputHDFFile outputMetadataFile.\n");
    printf("\n");
    exit(1);
  }

  printf("\n");
  printf("Input HDF-EOS file is %s.\n", argv[1]);
  printf("Output metadata file is %s.\n", argv[2]);
  printf("...\n");


  /* Get HDF file name. */
  hdfName=argv[1];
  
  /* Get meta file name. */
  metadataName=argv[2];
  metadataFile=fopen(metadataName, "w");

  /* Read metadata from hdf file. Write to temporary inventory and archive files. */
  status = readMetadata(hdfName, metadataFile, "CoreMetadata");
  status = readMetadata(hdfName, metadataFile, "coremetadata");
  status = readMetadata(hdfName, metadataFile, "ArchiveMetadata");
  status = readMetadata(hdfName, metadataFile, "archivemetadata");
  status = readMetadata(hdfName, metadataFile, "InventoryMetadata");
  status = readMetadata(hdfName, metadataFile, "inventorymetadata");
  status = readMetadata(hdfName, metadataFile, "Collection");
  status = readMetadata(hdfName, metadataFile, "collection");
  status = readMetadata(hdfName, metadataFile, "CollectionMetadata");
  status = readMetadata(hdfName, metadataFile, "collectionmetadata");
  status = readMetadata(hdfName, metadataFile, "StructMetadata");
  status = readMetadata(hdfName, metadataFile, "structmetadata");
  status = readMetadata(hdfName, metadataFile, "ProductMetadata");
  status = readMetadata(hdfName, metadataFile, "productmetadata");

  fclose(metadataFile);

  printf("Finished.\n");
  printf("\n");
}


/* Read metadata from hdfName and write temp file. */
intn readMetadata(char *hdfName, FILE *metadataFile, char *attr)
{
  intn i;
  int32 hdfFile;
  char attrname[100];

  hdfFile = SDstart(hdfName, DFACC_READ);
  if (hdfFile == -1) 
  {
    fprintf(stderr, "Unable to open HDF file %s \n", hdfName);
    exit(1);
  }

  /* First, try attr name alone */
  readAttr(hdfFile, metadataFile, attr);

  /* Now, try concatenating sequence numbers */
  for (i = 0; i <= 9; i++) 
  {
    sprintf(attrname, "%s.%d", attr, i);
    readAttr(hdfFile, metadataFile, attrname);
  }

  SDend(hdfFile);
  return (0);
}


/* Read metadata from hdfFile and write temp file. */
intn readAttr(int32 hdfFile, FILE *metadataFile, char *attr)
{
  intn status = 0;
  int32 attr_index, data_type, n_values;
  char tempString[50], *astring, *metadata, attr_name[MAX_NC_NAME];

  /* Look for attribute */
  attr_index = SDfindattr(hdfFile, attr);
  if (attr_index == -1) 
  {
    /* fprintf(stderr, "Unable to find attribute %s.\n", attr); */
    return (1);
  }
 
  /* Get size of attribute  */
  status = SDattrinfo(hdfFile, attr_index, attr_name, &data_type, &n_values);
  if (status == -1) 
  {
    fprintf(stderr, "SDattrinfo failed for existing attribute %s.\n", attr);
    return (1);
  }
 
  /* Allocate memory for attribute contents */
  metadata = (char *) malloc(n_values);
  if (metadata == NULL) 
  {
    fprintf(stderr, "Unable to allocate %d bytes for %s.\n", n_values, attr);
    return (1);
  }
 
  /* Read attribute */
  status = SDreadattr(hdfFile, attr_index, metadata);
  if (status == -1) 
  {
    fprintf(stderr, "Unable to read attribute %s.\n", attr);
    free(metadata);
    return (1);
  }

  fprintf(metadataFile, "%s", metadata);

  /* Clean up and return success */
  free(metadata);
  return (0);
}

